@@ -14,7 +14,7 @@ import numpy as np
 
 KNOWN_KEYS = {"nelec", "spin", "hf_occ", "schedule", "maxiter",
               "twodot_to_onedot", "twodot", "onedot", "zerodot", "sweep_tol",
-              "orbitals", "warmup", "nroots", "outputlevel", "prefix",
+              "orbitals", "integral_tol", "warmup", "nroots", "outputlevel", "prefix",
               "noreorder", "fiedler", "reorder", "gaopt", "nofiedler", "irrep_reorder",
               "num_thrds", "mkl_thrds", "mem", "intmem", "oh", "nonspinadapted",
               "onepdm", "fullrestart", "restart_onepdm", "restart_oh",
@@ -31,7 +31,7 @@ KNOWN_KEYS = {"nelec", "spin", "hf_occ", "schedule", "maxiter",
               "orbital_rotation", "delta_t", "target_t", "te_type",
               "read_mps_tags", "compression", "random_mps_init", "trans_mps_to_sz",
               "trans_mps_to_singlet_embedding", "trans_mps_from_singlet_embedding",
-              "copy_mps", "restart_copy_mps", "sample", "restart_sample", "resolve_twosz",
+              "copy_mps", "restart_copy_mps", "sample", "restart_sample", "resolve_twosz", "sample_phase", "sample_reference",
               "extrapolation", "cached_contraction", "singlet_embedding", "normalize_mps",
               "dmrgfci", "mrci", "mrcis", "mrcisd", "mrcisdt", "casci", "nevpt2", "nevpt2s",
               "nevpt2sd", "nevpt2-ijrs", "nevpt2-ij", "nevpt2-rs", "nevpt2-ijr",
@@ -40,12 +40,12 @@ KNOWN_KEYS = {"nelec", "spin", "hf_occ", "schedule", "maxiter",
               "mrrept2-rsi", "mrrept2-ir", "mrrept2-i", "mrrept2-r",
               "big_site", "stopt_dmrg", "stopt_compression", "stopt_sampling",
               "model", "k_symmetry", "k_irrep", "k_mod", "init_mps_center",
-              "use_complex", "real_density_matrix", "expt_algo_type",
-              "davidson_max_iter", "davidson_soft_max_iter", "one_body_parallel_rule",
+              "use_complex", "real_density_matrix", "expt_algo_type", "one_body_parallel_rule",
+              "davidson_max_iter", "davidson_soft_max_iter", "linear_soft_max_iter",
               "n_sub_sweeps", "complex_mps", "split_states", "trans_mps_to_complex",
               "use_general_spin", "trans_integral_to_spin_orbital", "store_wfn_spectra",
               "tran_bra_range", "tran_ket_range", "tran_triangular", "use_hybrid_complex",
-              "mem_ratio", "min_mpo_mem", "qc_mpo_type", "full_integral"}
+              "mem_ratio", "min_mpo_mem", "qc_mpo_type", "full_integral", "skip_inact_ext_sites"}
 
 REORDER_KEYS = {"noreorder", "fiedler", "reorder", "gaopt", "nofiedler",
                 "irrep_reorder"}
