@@ -905,6 +905,38 @@ template <typename FL> struct FCIDUMP {
             }
         }
     }
+    // Remove small integral elements
+    virtual FP truncate_small(FP tol) {
+        uint16_t n = n_sites();
+        FP error = 0.0;
+        for (auto &t : ts)
+            for (int i = 0; i < n; i++)
+                for (int j = 0; j < (t.general ? n : i + 1); j++)
+                    if (abs(t(i, j)) < tol)
+                        error += abs(t(i, j)), t(i, j) = 0;
+        for (auto &v : vgs)
+            for (int i = 0; i < n; i++)
+                for (int j = 0; j < n; j++)
+                    for (int k = 0; k < n; k++)
+                        for (int l = 0; l < n; l++)
+                            if (abs(v(i, j, k, l)) < tol)
+                                error += abs(v(i, j, k, l)), v(i, j, k, l) = 0;
+        for (auto &v : vabs)
+            for (int i = 0; i < n; i++)
+                for (int j = 0; j <= i; j++)
+                    for (int k = 0; k < n; k++)
+                        for (int l = 0; l <= k; l++)
+                            if (abs(v(i, j, k, l)) < tol)
+                                error += abs(v(i, j, k, l)), v(i, j, k, l) = 0;
+        for (auto &v : vs)
+            for (int i = 0, ij = 0; i < n; i++)
+                for (int j = 0; j <= i; j++, ij++)
+                    for (int k = 0, kl = 0; k <= i; k++)
+                        for (int l = 0; l <= k; l++, kl++)
+                            if (ij >= kl && abs(v(i, j, k, l)) < tol)
+                                error += abs(v(i, j, k, l)), v(i, j, k, l) = 0;
+        return error;
+    }
     // Remove integral elements that violate point group symmetry
     // orbsym: in XOR convention
     virtual FP symmetrize(const vector<uint8_t> &orbsym) {
@@ -1349,7 +1381,8 @@ template <typename FL> struct MRCISFCIDUMP : FCIDUMP<FL> {
     uint16_t n_inactive, n_virtual, n_active;
     MRCISFCIDUMP(const shared_ptr<FCIDUMP<FL>> &fcidump, uint16_t n_inactive,
                  uint16_t n_virtual)
-        : FCIDUMP<FL>(), prim_fcidump(fcidump), n_virtual(n_virtual),
+        : FCIDUMP<FL>(), prim_fcidump(fcidump), n_inactive(n_inactive),
+          n_virtual(n_virtual),
           n_active(fcidump->n_sites() - n_inactive - n_virtual) {
         params = fcidump->params;
     }
@@ -1366,22 +1399,14 @@ template <typename FL> struct MRCISFCIDUMP : FCIDUMP<FL> {
     FL v(uint16_t i, uint16_t j, uint16_t k, uint16_t l) const override {
         const uint16_t nocc = n_inactive + n_active;
         const int cnt = (i >= nocc) + (j >= nocc) + (k >= nocc) + (l >= nocc);
-        if (cnt <= 1 || (cnt == 2 && (i >= nocc) == (j >= nocc) ||
-                         (i >= nocc) == (l >= nocc)))
-            return prim_fcidump->v(i, j, k, l);
-        else
-            return 0;
+        return cnt <= 2 ? prim_fcidump->v(i, j, k, l) : 0;
     }
     // Two-electron integral element (SZ)
     FL v(uint8_t sl, uint8_t sr, uint16_t i, uint16_t j, uint16_t k,
          uint16_t l) const override {
         const uint16_t nocc = n_inactive + n_active;
         const int cnt = (i >= nocc) + (j >= nocc) + (k >= nocc) + (l >= nocc);
-        if (cnt <= 1 || (cnt == 2 && (i >= nocc) == (j >= nocc) ||
-                         (i >= nocc) == (l >= nocc)))
-            return prim_fcidump->v(sl, sr, i, j, k, l);
-        else
-            return 0;
+        return cnt <= 2 ? prim_fcidump->v(sl, sr, i, j, k, l) : 0;
     }
     FL e() const override { return prim_fcidump->e(); }
     void deallocate() override {}
